/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.cropprotection.clm;

import com.sun.jersey.api.client.UniformInterfaceException;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.ClmCrop;
import nl.cloudfarming.client.model.CommonModelService;
import nl.cloudfarming.client.model.crop.Crop;
import nl.cloudfarming.client.model.CropProtection;
import nl.cloudfarming.client.model.CropProtectionMessage;
import nl.cloudfarming.client.model.crop.CropService;
import nl.cloudfarming.client.stock.CropProtectionService;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 * Service for the cropProtection CLM web service
 * Because of the serviceProvider it is initialized once.
 * On initialization will load all the CropProtections from the web service and update or persist them
 * @author Merijn Zengers
 */
@ServiceProvider(service = CropProtectionService.class)
public final class CropProtectionClmService extends CommonModelService implements CropProtectionService {

    private CropProtectionMarshaller marshaller = new CropProtectionMarshaller();
    private CropService cropService = Lookup.getDefault().lookup(CropService.class);
    private static final AppLogger LOG = AppLogFactory.getLogger(CommonModelService.class);
    private static final String NO_CONNECTION_MESSAGE = NbBundle.getMessage(CropProtecionClmModule.class, "no_connection_message");


    /**
     * Find the CropProtections for the crop from the web service
     * call the method updateCropProtection. Which will check if it's already persisted  
     */
    @Override
    public List<CropProtection> getCropProtections() {
        
        AuthorizationServiceLayer client = new AuthorizationServiceLayer();
        ArrayList<CropProtection> returnList = new ArrayList<CropProtection>(); 
        try{
        String name = "%";
        String response = client.getCrop(String.class, name);
        
        CropProtectionMessage cpm = marshaller.unMarshall(response);
        for (CropProtection cropProtection : cpm.getCropProtection()) {
            Crop crop = translateClmCropToCrop(cropProtection.getClmCrop());
                if (crop != null) {
                    cropProtection.setCrop(crop);
                    returnList.add(cropProtection);
                }
        }

        client.close();
        } catch(UniformInterfaceException ex){
            LOG.info(NO_CONNECTION_MESSAGE);
        } catch(com.sun.jersey.api.client.ClientHandlerException e){
            LOG.info(NO_CONNECTION_MESSAGE);
        }
        return returnList;
    }

    /**
     * TransLate the CLM crop name to a crop entity which can be used in de application
     * @param clmCrop the CLM crop to translate
     * @return the Crop which is found, If not found null is returned.
     */
    protected Crop translateClmCropToCrop(ClmCrop clmCrop) {
        try {
            String key = NbBundle.getMessage(CropProtecionClmModule.class, clmCrop.getName());
            Crop crop = cropService.findCrop(key);
            return crop;
        } catch (java.util.MissingResourceException ex) {
            StringBuilder errorBuilder = new StringBuilder("Could not translate clm crop with name ");
            errorBuilder.append(clmCrop.getName());
            errorBuilder.append("to crop");
            LOG.info(errorBuilder.toString());
        }
        return null;
    }

    @Override
    protected String getModuleName() {
        return CropProtecionClmModule.MODULE_NAME;
    }

    @Override
    protected void initExplorer() {
    }
}
